// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect-cli-config
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

'use strict';

var _ = require('lodash');
var conf = require('apiconnect-config');
var g = require('strong-globalize')();
var logger = require('apiconnect-cli-logger');

var config = conf.loadConfig({ shouldParseUris: false });

var validConfigNames = [
  'accessibility-mode',
  'app',
  'catalog',
  'microgateway',
  'log-level',
  'org',
  'template-path',
  'template-default-api',
  'template-default-product',
  'trust-store',
  'client-certificate',
  'client-certificate-password',
  'space',
];

module.exports.list = function(options) {
  var data = config.get(/.*/, _getStore(options));
  print(data);
};

function print(data) {
  _.forEach(data, function(v, k) {
    if (_.includes(validConfigNames, k)) {
      logger.writeln(g.f('%s: %s', k, v));
    }
  });
};

module.exports.getConfig = function(name, options) {
  if (!_.includes(validConfigNames, name)) {
    throw new Error(g.f('%s is not a valid configuration variable', name));
  }
  var data = config.get(name, _getStore(options));
  if (!data.hasOwnProperty(name)) {
    throw new Error(g.f('Configuration %s is not set', name));
  }
  logger.writeln(g.f('%s: %s', name, data[name]));
};

module.exports.setConfig = function(kv, otherKVs, options) {
  kv = _.concat([], kv, otherKVs);
  kv = _.fromPairs(_.map(kv, function(v) {
    var param = v.split('=');
    if (param.length === 1) {
      throw new Error(g.f('Invalid argument format `name=value`'));
    }
    if (!param[1]) {
      throw new Error(g.f('Invalid argument value specified.'));
    }
    return param;
  }));

  var invalidConfigNames = _.remove(_.keys(kv), function(name) {
    return !_.includes(validConfigNames, name);
  });
  if (invalidConfigNames.length > 0) {
    throw new Error(g.f('%s is not a valid configuration variable', invalidConfigNames[0]));
  }

  kv = config.set(kv, _getStore(options, conf.PROJECT_STORE));
  print(kv);
};

module.exports.deleteConfig = function(name, options) {
  var store = _getStore(options, conf.PROJECT_STORE);
  var data = config.get(/.*/, store);
  if (!data.hasOwnProperty(name) || !_.includes(validConfigNames, name)) {
    logger.error(g.f('Configuration %s is not set', name));
  } else {
    config.delete(name, store);
    logger.writeln(g.f('Deleted %s: %s', name, data[name]));
  }
};

module.exports.clearConfig = function(options) {
  var store = _getStore(options, conf.PROJECT_STORE);
  var data = config.get(/.*/, store);
  _.forEach(data, function(_v, _k) {
    // clean out only user settable configs
    if (!_.includes(validConfigNames, _k)) {
      return;
    }
    config.delete(_k, store);
    logger.writeln(g.f('Deleted %s: %s', _k, data[_k]));
  });
};

function _getStore(options, defaultStore) {
  if (options.global) {
    return conf.USER_STORE;
  }
  if (options.local) {
    return conf.PROJECT_STORE;
  }
  return defaultStore;
};
